@@ -34,6 +34,7 @@ import redis
 import requests
 from aiohttp import web
 from dotenv import load_dotenv
+from expiringdict import ExpiringDict  # type: ignore
 from sqlalchemy import exc
 from sqlalchemy.orm.session import Session
 from sqlalchemy.sql import text
@@ -44,6 +45,7 @@ from spellbot.constants import (
     ADMIN_ROLE,
     CREATE_ENDPOINT,
     DEFAULT_GAME_SIZE,
+    ICO_URL,
     INVITE_LINK,
     THUMB_URL,
     VOTE_LINK,
@@ -62,15 +64,14 @@ from spellbot.data import (
     UserTeam,
 )
 from spellbot.operations import (
-    ChannelType,
     safe_clear_reactions,
     safe_create_category_channel,
     safe_create_invite,
     safe_create_voice_channel,
     safe_delete_channel,
-    safe_delete_message,
     safe_edit_message,
     safe_fetch_channel,
+    safe_fetch_guild,
     safe_fetch_message,
     safe_fetch_user,
     safe_react_error,
@@ -311,21 +312,10 @@ class SpellBot(discord.Client):
         self.token = token or ""
         self.auth = auth or ""
         self.mock_games = mock_games
+        self.channel_lock_cache = ExpiringDict(max_len=100, max_age_seconds=3600)  # 1 hr
 
         # Connect to Redis Cloud if we have a URL for it.
-        self.metrics_db: Optional[redis.Redis] = None
-        if redis_url:
-            url = parse.urlparse(redis_url)
-            if url.hostname and url.port:
-                try:
-                    self.metrics_db = redis.Redis(  # type: ignore
-                        host=url.hostname,
-                        port=url.port,
-                        password=url.password,
-                        health_check_interval=30,  # type: ignore
-                    )
-                except redis.exceptions.RedisError as e:
-                    logger.exception("redis error: %s", e)
+        self.metrics_db = self._create_metrics_db(redis_url)
 
         # We have to make sure that DB_DIR exists before we try to create
         # the database as part of instantiating the Data object.
@@ -358,6 +348,33 @@ class SpellBot(discord.Client):
         finally:
             session.close()
 
+    @asynccontextmanager
+    async def channel_lock(self, channel_xid: int) -> AsyncGenerator[None, None]:
+        """Aquire an async lock based on the given channel id."""
+        lock = self.channel_lock_cache.get(channel_xid, asyncio.Lock())
+        self.channel_lock_cache[channel_xid] = lock
+        async with lock:  # type: ignore
+            yield
+
+    def _create_metrics_db(
+        self, redis_url: Optional[str]
+    ) -> Optional[redis.Redis]:  # pragma: no cover
+        if not redis_url:
+            return None
+
+        url = parse.urlparse(redis_url)
+        if url.hostname and url.port:
+            try:
+                return redis.Redis(  # type: ignore
+                    host=url.hostname,
+                    port=url.port,
+                    password=url.password,
+                    health_check_interval=30,  # type: ignore
+                )
+            except redis.exceptions.RedisError as e:
+                logger.exception("redis error: %s", e)
+        return None
+
     def _begin_background_tasks(
         self, loop: asyncio.AbstractEventLoop
     ) -> None:  # pragma: no cover
@@ -385,27 +402,39 @@ class SpellBot(discord.Client):
     async def cleanup_expired_games(self) -> None:
         """Culls games older than the given window of minutes."""
         logger.info("starting expired games cleanup task...")
+
+        to_delete_args = []
         async with self.session() as session:
             expired = Game.expired(session)
             for game in expired:
-                await self.try_to_delete_message(game)
+                if not game.is_expired():
+                    continue
+
+                if game.guild_xid and game.channel_xid and game.message_xid:
+                    to_delete_args.append(
+                        {
+                            "guild_xid": game.guild_xid,
+                            "channel_xid": game.channel_xid,
+                            "message_xid": game.message_xid,
+                        }
+                    )
+
                 for user in game.users:
-                    discord_user = await safe_fetch_user(self, user.xid)
-                    if discord_user:
-                        await safe_send_user(
-                            discord_user,
-                            s(
-                                "expired",
-                                reply=f"<@{discord_user.id}>",
-                                window=game.server.expire,
-                            ),
-                        )
-                    user.game_id = None
+                    # Make sure the user is still waiting and still in the
+                    # game that's being deleted, they could be in a new
+                    # game now due to how async processing works.
+                    if user.waiting and user.game_id == game.id:
+                        user.game_id = None
+
                 # cascade delete tag associations
                 game.tags = []  # type: ignore
                 session.delete(game)
             session.commit()
 
+        for args in to_delete_args:
+            async with self.channel_lock(args["channel_xid"]):
+                await self.try_to_delete_message(**args)
+
     def cleanup_old_voice_channels_task(
         self, loop: asyncio.AbstractEventLoop
     ) -> None:  # pragma: no cover
@@ -478,7 +507,7 @@ class SpellBot(discord.Client):
 
         loop.create_task(task())
 
-    async def update_metrics(self) -> None:
+    async def update_metrics(self) -> None:  # pragma: no cover
         if not self.metrics_db:
             return
 
@@ -521,10 +550,12 @@ class SpellBot(discord.Client):
 
     async def setup_voice(self, session: Session, game: Game) -> None:
         """Adds voice information to the game object. DOES NOT COMMIT!"""
-        if not game.server.create_voice:
+        # NOTE: See the test_simultaneous_signup() test for more details about
+        #       how this function has caused issues in the past.
+        if not game.server.create_voice or game.voice_channel_xid:
             return
 
-        category = await self.ensure_server_voice_category(session, game.server)
+        category = await self.ensure_available_voice_category(game.server)
 
         # if category is None, then we'll simply create a non-categorized channel
         voice_channel = await safe_create_voice_channel(
@@ -565,48 +596,69 @@ class SpellBot(discord.Client):
             session.commit()
         return cast(Server, server)
 
-    async def ensure_server_voice_category(
-        self, session: Session, server: Server
-    ) -> Optional[discord.CategoryChannel]:
-        category: Optional[ChannelType] = None
-
+    async def ensure_available_voice_category(
+        self, server: Server
+    ) -> Optional[discord.CategoryChannel]:  # pragma: no cover
+        # TODO: Test for this need to be written, but for now just ignore it.
         if not server.create_voice:
             return None
 
-        if server.voice_category_xid:
-            category = await safe_fetch_channel(
-                self, server.voice_category_xid, server.guild_xid
+        guild = await safe_fetch_guild(self, server.guild_xid)
+        if not guild:
+            return None
+
+        PREFIX = "SpellBot Voice Channels"
+
+        def category_num(cat: discord.CategoryChannel) -> int:
+            return 0 if cat.name == PREFIX else int(cat.name[24:]) - 1
+
+        def category_name(i: int) -> str:
+            return PREFIX if i == 0 else f"{PREFIX} {i + 1}"
+
+        available: Optional[discord.CategoryChannel] = None
+        full: List[discord.CategoryChannel] = []
+        for i, cat in enumerate(
+            sorted(
+                (c for c in guild.categories if c.name.startswith(PREFIX)),
+                key=category_num,
             )
-        if category:
-            return cast(discord.CategoryChannel, category)
+        ):
+            if i != category_num(cat):
+                break  # there's a missing category, we need to re-create it
+            if len(cat.channels) < 50:
+                available = cat
+                break  # we found an available channel, use it
+            else:
+                full.append(cat)  # keep track of how many full channels there are
+
+        if available:
+            return cast(discord.CategoryChannel, available)
 
         category = await safe_create_category_channel(
-            self, server.guild_xid, "SpellBot Voice Channels"
+            self, server.guild_xid, category_name(len(full))
         )
-        if category:
-            server.voice_category_xid = category.id  # type: ignore
-            session.commit()
-            return cast(discord.CategoryChannel, category)
+        if not category:
+            return None
 
-        return None
+        return cast(discord.CategoryChannel, category)
 
-    async def try_to_delete_message(self, game: Game) -> None:
+    async def try_to_delete_message(
+        self, guild_xid: int, channel_xid: int, message_xid: int
+    ) -> None:
         """Attempts to remove a ✋ from the given game message for the given user."""
-        if not game.channel_xid:
-            return
-
-        chan = await safe_fetch_channel(self, game.channel_xid, game.guild_xid)
+        chan = await safe_fetch_channel(self, channel_xid, guild_xid)
         if not chan:
             return
 
-        if not game.message_xid:
-            return
-
-        post = await safe_fetch_message(chan, game.message_xid, game.guild_xid)
+        post = await safe_fetch_message(chan, message_xid, guild_xid)
         if not post:
             return
 
-        await safe_delete_message(post)
+        # NOTE: Deleting the whole post ended up being confusing by invalidating
+        #       hyperlinks to game posts. Instead, let's just empty the post contents.
+        # await safe_delete_message(post)
+        await safe_edit_message(post, content=s("deleted_game"), embed=None)
+        await safe_clear_reactions(post)
 
     async def try_to_update_game(self, game) -> None:
         """Attempts to update the embed for a game."""
@@ -636,6 +688,10 @@ class SpellBot(discord.Client):
             params.insert(0, request[5:])
             request = "power"
 
+        # Ignore !help because so many other bots use it, use `!spellbot help` instead.
+        if request.startswith("help"):
+            return
+
         matching = [command for command in self.commands if command.startswith(request)]
         if not matching:
             await message.channel.send(
@@ -698,7 +754,10 @@ class SpellBot(discord.Client):
         if not message:
             return
 
-        async with self.session() as session:
+        # NOTE: This and the code that handles !lfg/!find commands is behind an async
+        #       channel lock to prevent more than one person per guild per channel
+        #       concurrently interleaving processing within this critical section.
+        async with self.session() as session, self.channel_lock(payload.channel_id):
             assert payload.guild_id is not None
             server = self.ensure_server_exists(session, payload.guild_id)
 
@@ -848,7 +907,7 @@ class SpellBot(discord.Client):
     #
     # Where [foo] indicates foo is optional and <bar> indicates bar is required.
 
-    @command(allow_dm=True, help_group="Commands for Players")
+    # @command(allow_dm=True, help_group="Commands for Players")
     async def help(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
     ) -> None:
@@ -927,7 +986,7 @@ class SpellBot(discord.Client):
         embed.description = (
             "_The Discord bot for [SpellTable](https://www.spelltable.com/)._\n"
             "\n"
-            f"Use the command `{prefix}help` for usage details. "
+            f"Use the command `{prefix}spellbot help` for usage details. "
             "Having issues with SpellBot? "
             "Please [report bugs](https://github.com/lexicalunit/spellbot/issues)!\n"
             "\n"
@@ -953,7 +1012,7 @@ class SpellBot(discord.Client):
     async def _validate_mentions_size(
         self, msg: discord.Message, mentions: List[Any], size: int
     ) -> bool:
-        if len(mentions) + 1 >= size:
+        if len(mentions) >= size:
             user_xid = cast(discord.User, msg.author).id
             await msg.channel.send(s("lfg_too_many_mentions", reply=f"<@{user_xid}>"))
             return False
@@ -983,8 +1042,12 @@ class SpellBot(discord.Client):
         if not post:
             return None
         link = post_link(game.server.guild_xid, msg.channel.id, game.message_xid)
-        resp = s("play_found", reply=f"<@{user.xid}>", link=link)
-        await msg.channel.send(resp)
+        embed = discord.Embed()
+        embed.set_thumbnail(url=ICO_URL)
+        embed.set_author(name=s("play_found_title"))
+        embed.description = s("play_found_desc", reply=f"<@{user.xid}>", link=link)
+        embed.color = discord.Color(0x5A3EFD)
+        await msg.channel.send(embed=embed)
         return post
 
     async def _add_user_to_game(self, session: Session, user: User, game: Game) -> None:
@@ -1033,18 +1096,41 @@ class SpellBot(discord.Client):
         else:  # game *definitely* isn't ready yet
             await safe_edit_message(post, embed=game.to_embed())
 
+    async def _call_attention_to_game(self, msg: discord.Message, game: Game) -> bool:
+        if not game.message_xid:
+            return False
+
+        post = await safe_fetch_message(msg.channel, game.message_xid, game.guild_xid)
+        if not post:
+            return False
+
+        link = post_link(game.server.guild_xid, msg.channel.id, game.message_xid)
+        embed = discord.Embed()
+        embed.set_thumbnail(url=ICO_URL)
+        remaining = int(game.size) - len(cast(List[User], game.users))
+        plural = "s" if remaining > 1 else ""
+        embed.set_author(name=s("call_attn_title", remaining=remaining, plural=plural))
+        embed.description = s("call_attn_desc", link=link)
+        embed.color = discord.Color(0x5A3EFD)
+        await msg.channel.send(embed=embed)
+        return True
+
     async def _play_helper(
         self,
         session: Session,
         prefix: str,
         params: List[str],
         message: discord.Message,
-        create_new_game: bool,
     ) -> None:
         server = self.ensure_server_exists(session, message.channel.guild.id)
         user = self.ensure_user_exists(session, message.author)
         mentions = message.mentions if message.channel.type != "private" else []
 
+        if user.waiting and user.game.channel_xid == message.channel.id:
+            await self._call_attention_to_game(message, user.game)
+            await safe_react_ok(message)
+            return
+
         opts = parse_opts(params)
         size: Optional[int] = opts["size"]
         tag_names: List[str] = opts["tags"]
@@ -1070,23 +1156,8 @@ class SpellBot(discord.Client):
             return
         tags = Tag.create_many(session, tag_names)
 
-        if mentioned_users:
-            other = mentioned_users[0]
-            for mentioned_user in mentioned_users:
-                if not mentioned_user.waiting or mentioned_user.game != other.game:
-                    await message.channel.send(
-                        s("lfg_mentions_different_games", reply=f"<@{user.xid}>")
-                    )
-                    await safe_react_error(message)
-                    return
-            await self._remove_user_from_game(session, user)
-            await self._add_user_to_game(session, user, other.game)
-            game_post = await self._respond_found_game(message, user, other.game)
-            if not game_post:
-                game_post = await self._post_new_game(session, message, other.game)
-            await self._update_or_start_game(session, other.game, game_post)
-            await safe_react_ok(message)
-            return
+        # you can only add mentioned users to the game if they're not waiting
+        free_users = [u for u in mentioned_users if not u.waiting]
 
         new_game = False
         game = Game.find_existing(
@@ -1094,18 +1165,13 @@ class SpellBot(discord.Client):
             server=server,
             channel_xid=message.channel.id,
             size=size,
-            seats=1,
+            seats=1 + len(free_users),
             tags=tags,
             system=system,
             power=user.power,
         )
 
         if not game:
-            if not create_new_game:
-                await message.channel.send(s("game_not_found", reply=f"<@{user.xid}>"))
-                await safe_react_error(message)
-                return
-
             now = datetime.utcnow()
             expires_at = now + timedelta(minutes=server.expire)
             new_game = True
@@ -1122,6 +1188,8 @@ class SpellBot(discord.Client):
 
         await self._remove_user_from_game(session, user)
         await self._add_user_to_game(session, user, game)
+        for free_user in free_users:
+            await self._add_user_to_game(session, free_user, game)
 
         post: Optional[discord.Message] = None
         if new_game:
@@ -1156,16 +1224,11 @@ class SpellBot(discord.Client):
         are set on your game when they are looking for games to join.
         & [size:N] [@player-1] [@player-2] ... [~tag-1] [~tag-2] ...
         """
-        await self._play_helper(session, prefix, params, message, create_new_game=True)
-
-    @command(allow_dm=False, help_group="Commands for Players")
-    async def find(
-        self, session: Session, prefix: str, params: List[str], message: discord.Message
-    ) -> None:
-        """
-        Works exactly like `!lfg` except that it will NOT create a new game.
-        """
-        await self._play_helper(session, prefix, params, message, create_new_game=False)
+        # NOTE: This and the code to handle message reactions is behind an async
+        #       channel lock to prevent more than one person per guild per channel
+        #       concurrently interleaving processing within this critical section.
+        async with self.channel_lock(message.channel.id):
+            await self._play_helper(session, prefix, params, message)
 
     async def _verify_command_fest_report(
         self,
