@@ -91,10 +91,19 @@ async def safe_fetch_message(
         return None
     try:
         return await channel.fetch_message(message_xid)
+    except discord.errors.NotFound as e:
+        # An admin could have deleted the message. This bot recovers from
+        # this situation by simply creating a new game post. This happens
+        # often enough that having it be a warning log is too verbose.
+        logger.debug(
+            "debug: discord (guild %s): could not fetch message: %s",
+            guild_xid,
+            e,
+        )
+        return None
     except (
         discord.errors.Forbidden,
         discord.errors.HTTPException,
-        discord.errors.NotFound,
     ) as e:
         logger.exception(
             "warning: discord (guild %s): could not fetch message: %s",
@@ -174,8 +183,15 @@ async def safe_delete_message(message: discord.Message) -> None:
 async def safe_send_user(user: discord.User, *args, **kwargs) -> None:
     try:
         await user.send(*args, **kwargs)
+    except discord.errors.Forbidden as e:
+        # User may have the bot blocked or they may have DMs only allowed for friends.
+        # Generally speaking, we can safely ignore this sort of error.
+        logger.debug(
+            "debug: discord (DM): could not send message to user (%s): %s",
+            str(user),
+            e,
+        )
     except (
-        discord.errors.Forbidden,
         discord.errors.HTTPException,
         discord.errors.InvalidArgument,
     ) as e:
@@ -260,3 +276,23 @@ async def safe_create_invite(
             e,
         )
         return None
+
+
+async def safe_fetch_guild(
+    client: discord.Client, guild_xid: int
+) -> Optional[discord.Guild]:
+    guild = client.get_guild(guild_xid)
+    if guild:
+        return guild
+    try:
+        return await client.fetch_guild(guild_xid)
+    except (
+        discord.errors.Forbidden,
+        discord.errors.HTTPException,
+    ) as e:
+        logger.exception(
+            "warning: discord (guild %s): could not fetch guild: %s",
+            guild_xid,
+            e,
+        )
+        return None
